package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.SafeRuleSubBo;
import org.eco.als.domain.vo.SafeRuleSubImportVo;
import org.eco.als.domain.vo.SafeRuleSubVo;
import org.eco.als.service.ISafeRuleSubService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * 保障规则子Controller
 *
 * @author wgk
 * @date 2024-07-24
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/safeRuleSub")
public class SafeRuleSubController extends BaseController {
    @Resource
    private ISafeRuleSubService safeRuleSubService;

    /**
     * 查询保障规则子列表
     */
    @SaCheckPermission("als:safeRuleSub:list")
    @GetMapping("/list")
    public CommonResult<PageResult<SafeRuleSubVo>> list(SafeRuleSubBo safeRuleSubBo) {
        return CommonResult.success(safeRuleSubService.selectPage(safeRuleSubBo));
    }

    /**
     * 导出保障规则子列表
     */
    @SaCheckPermission("als:safeRuleSub:export")
    @Log(title = "保障规则子", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(SafeRuleSubBo safeRuleSubBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<SafeRuleSubVo> list = safeRuleSubService.selectList(safeRuleSubBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        safeRuleSubService.asyncExport(list, "保障规则子", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:safeRuleSub:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "保障规则子", SafeRuleSubImportVo.class, response);
    }

    /**
     * 导入保障规则子列表
     */
    @Log(title = "保障规则子", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:safeRuleSub:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        safeRuleSubService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取保障规则子详细信息
     */
    @SaCheckPermission("als:safeRuleSub:query")
    @GetMapping(value = "/{id}")
    public CommonResult<SafeRuleSubVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(safeRuleSubService.selectById(id));
    }

    /**
     * 新增保障规则子
     */
    @SaCheckPermission("als:safeRuleSub:add")
    @Log(title = "保障规则子", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody SafeRuleSubBo safeRuleSubBo) {
        boolean inserted = safeRuleSubService.insert(safeRuleSubBo);
        if (!inserted) {
            return CommonResult.fail("新增保障规则子记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改保障规则子
     */
    @SaCheckPermission("als:safeRuleSub:edit")
    @Log(title = "保障规则子", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody SafeRuleSubBo safeRuleSubBo) {
        boolean updated = safeRuleSubService.update(safeRuleSubBo);
        if (!updated) {
            return CommonResult.fail("修改保障规则子记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除保障规则子
     */
    @SaCheckPermission("als:safeRuleSub:remove")
    @Log(title = "保障规则子", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = safeRuleSubService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除保障规则子记录失败!");
        }
        return CommonResult.success();
    }
}
