package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.SparePartsBo;
import org.eco.als.domain.vo.SparePartsImportVo;
import org.eco.als.domain.vo.SparePartsVo;
import org.eco.als.service.ISparePartsService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * 部件Controller
 *
 * @author wgk
 * @date 2024-07-25
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/spareParts")
public class SparePartsController extends BaseController {
    @Resource
    private ISparePartsService sparePartsService;

    /**
     * 查询部件列表
     */
    @SaCheckPermission("als:spareParts:list")
    @GetMapping("/list")
    public CommonResult<PageResult<SparePartsVo>> list(SparePartsBo sparePartsBo) {
        return CommonResult.success(sparePartsService.selectPage(sparePartsBo));
    }

    /**
     * 导出部件列表
     */
    @SaCheckPermission("als:spareParts:export")
    @Log(title = "部件", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(SparePartsBo sparePartsBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<SparePartsVo> list = sparePartsService.selectList(sparePartsBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        sparePartsService.asyncExport(list, "部件", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:spareParts:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "部件", SparePartsImportVo.class, response);
    }

    /**
     * 导入部件列表
     */
    @Log(title = "部件", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:spareParts:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        sparePartsService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取部件详细信息
     */
    @SaCheckPermission("als:spareParts:query")
    @GetMapping(value = "/{id}")
    public CommonResult<SparePartsVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(sparePartsService.selectById(id));
    }

    /**
     * 新增部件
     */
    @SaCheckPermission("als:spareParts:add")
    @Log(title = "部件", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody SparePartsBo sparePartsBo) {
        boolean inserted = sparePartsService.insert(sparePartsBo);
        if (!inserted) {
            return CommonResult.fail("新增部件记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改部件
     */
    @SaCheckPermission("als:spareParts:edit")
    @Log(title = "部件", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody SparePartsBo sparePartsBo) {
        boolean updated = sparePartsService.update(sparePartsBo);
        if (!updated) {
            return CommonResult.fail("修改部件记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除部件
     */
    @SaCheckPermission("als:spareParts:remove")
    @Log(title = "部件", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = sparePartsService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除部件记录失败!");
        }
        return CommonResult.success();
    }
}
