package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.SafeRule;
import org.eco.als.domain.bo.SafeRuleBo;
import org.eco.als.domain.vo.SafeRuleImportVo;
import org.eco.als.domain.vo.SafeRuleVo;
import org.eco.als.listener.SafeRuleImportListener;
import org.eco.als.mapper.SafeRuleMapper;
import org.eco.als.service.ISafeRuleService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.SafeRuleTableDef.SAFE_RULE;

/**
 * 保障规则Service业务层处理
 *
 * @author wgk
 * @date 2024-07-24
 */
@Service
@Slf4j
public class SafeRuleServiceImpl extends BaseServiceImpl<SafeRuleMapper, SafeRule> implements ISafeRuleService {
    @Resource
    private SafeRuleMapper safeRuleMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(SAFE_RULE);
    }

    private QueryWrapper buildQueryWrapper(SafeRuleBo safeRuleBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(SAFE_RULE.AIRCRAFT_MODEL.eq
            (safeRuleBo.getAircraftModel()));
        queryWrapper.and(SAFE_RULE.DURATION.eq
            (safeRuleBo.getDuration()));
        queryWrapper.and(SAFE_RULE.TRAIN_EXERCISE.eq
            (safeRuleBo.getTrainExercise()));
        queryWrapper.and(SAFE_RULE.STATUS.eq
            (safeRuleBo.getStatus()));

        return queryWrapper;
    }

    /**
     * 查询保障规则
     *
     * @param id 保障规则主键
     * @return 保障规则
     */
    @Override
    public SafeRuleVo selectById(Long id) {
        return this.getOneAs(query().where(SAFE_RULE.ID.eq(id)), SafeRuleVo.class);

    }


    /**
     * 查询保障规则列表
     *
     * @param safeRuleBo 保障规则Bo
     * @return 保障规则集合
     */
    @Override
    public List<SafeRuleVo> selectList(SafeRuleBo safeRuleBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(safeRuleBo);
        return this.listAs(queryWrapper, SafeRuleVo.class);
    }

    /**
     * 分页查询保障规则列表
     *
     * @param safeRuleBo 保障规则Bo
     * @return 分页保障规则集合
     */
    @Override
    public PageResult<SafeRuleVo> selectPage(SafeRuleBo safeRuleBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(safeRuleBo);
        Page<SafeRuleVo> page = this.pageAs(PageQuery.build(), queryWrapper, SafeRuleVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增保障规则
     *
     * @param safeRuleBo 保障规则Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(SafeRuleBo safeRuleBo) {
        SafeRule safeRule = MapstructUtils.convert(safeRuleBo, SafeRule.class);

        return this.save(safeRule);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增保障规则，前台提供主键值，一般用于导入的场合
     *
     * @param safeRuleBo 保障规则Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(SafeRuleBo safeRuleBo) {
        SafeRule safeRule = MapstructUtils.convert(safeRuleBo, SafeRule.class);


        return safeRuleMapper.insertWithPk(safeRule) > 0;//前台传来主键值
    }

    /**
     * 修改保障规则
     *
     * @param safeRuleBo 保障规则Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(SafeRuleBo safeRuleBo) {
        SafeRule safeRule = MapstructUtils.convert(safeRuleBo, SafeRule.class);
        if (ObjectUtil.isNotNull(safeRule) && ObjectUtil.isNotNull(safeRule.getId())) {
            return this.updateById(safeRule);
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, SafeRuleImportVo.class, new SafeRuleImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<SafeRuleVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, SafeRuleVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除保障规则
     *
     * @param ids 需要删除的保障规则主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
