package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.SafeRuleSub;
import org.eco.als.domain.bo.SafeRuleSubBo;
import org.eco.als.domain.vo.SafeRuleSubImportVo;
import org.eco.als.domain.vo.SafeRuleSubVo;
import org.eco.als.listener.SafeRuleSubImportListener;
import org.eco.als.mapper.SafeRuleSubMapper;
import org.eco.als.service.ISafeRuleSubService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.SafeRuleSubTableDef.SAFE_RULE_SUB;

/**
 * 保障规则子Service业务层处理
 *
 * @author wgk
 * @date 2024-07-24
 */
@Service
@Slf4j
public class SafeRuleSubServiceImpl extends BaseServiceImpl<SafeRuleSubMapper, SafeRuleSub> implements ISafeRuleSubService {
    @Resource
    private SafeRuleSubMapper safeRuleSubMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(SAFE_RULE_SUB);
    }

    private QueryWrapper buildQueryWrapper(SafeRuleSubBo safeRuleSubBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(SAFE_RULE_SUB.AIRCRAFT_MODEL.eq
            (safeRuleSubBo.getAircraftModel()));
        queryWrapper.and(SAFE_RULE_SUB.AIRCRAFT_GROUP.eq
            (safeRuleSubBo.getAircraftGroup()));
        queryWrapper.and(SAFE_RULE_SUB.DURATION.eq
            (safeRuleSubBo.getDuration()));
        queryWrapper.and(SAFE_RULE_SUB.DEVICE.eq
            (safeRuleSubBo.getDevice()));
        queryWrapper.and(SAFE_RULE_SUB.DEVICE_COUNT.eq
            (safeRuleSubBo.getDeviceCount()));
        queryWrapper.and(SAFE_RULE_SUB.UNIT.eq
            (safeRuleSubBo.getUnit()));
        queryWrapper.and(SAFE_RULE_SUB.STATUS.eq
            (safeRuleSubBo.getStatus()));

        return queryWrapper;
    }

    /**
     * 查询保障规则子
     *
     * @param id 保障规则子主键
     * @return 保障规则子
     */
    @Override
    public SafeRuleSubVo selectById(Long id) {
        return this.getOneAs(query().where(SAFE_RULE_SUB.ID.eq(id)), SafeRuleSubVo.class);

    }


    /**
     * 查询保障规则子列表
     *
     * @param safeRuleSubBo 保障规则子Bo
     * @return 保障规则子集合
     */
    @Override
    public List<SafeRuleSubVo> selectList(SafeRuleSubBo safeRuleSubBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(safeRuleSubBo);
        return this.listAs(queryWrapper, SafeRuleSubVo.class);
    }

    /**
     * 分页查询保障规则子列表
     *
     * @param safeRuleSubBo 保障规则子Bo
     * @return 分页保障规则子集合
     */
    @Override
    public PageResult<SafeRuleSubVo> selectPage(SafeRuleSubBo safeRuleSubBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(safeRuleSubBo);
        Page<SafeRuleSubVo> page = this.pageAs(PageQuery.build(), queryWrapper, SafeRuleSubVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增保障规则子
     *
     * @param safeRuleSubBo 保障规则子Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(SafeRuleSubBo safeRuleSubBo) {
        SafeRuleSub safeRuleSub = MapstructUtils.convert(safeRuleSubBo, SafeRuleSub.class);

        return this.save(safeRuleSub);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增保障规则子，前台提供主键值，一般用于导入的场合
     *
     * @param safeRuleSubBo 保障规则子Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(SafeRuleSubBo safeRuleSubBo) {
        SafeRuleSub safeRuleSub = MapstructUtils.convert(safeRuleSubBo, SafeRuleSub.class);


        return safeRuleSubMapper.insertWithPk(safeRuleSub) > 0;//前台传来主键值
    }

    /**
     * 修改保障规则子
     *
     * @param safeRuleSubBo 保障规则子Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(SafeRuleSubBo safeRuleSubBo) {
        SafeRuleSub safeRuleSub = MapstructUtils.convert(safeRuleSubBo, SafeRuleSub.class);
        if (ObjectUtil.isNotNull(safeRuleSub) && ObjectUtil.isNotNull(safeRuleSub.getId())) {
            return this.updateById(safeRuleSub);
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, SafeRuleSubImportVo.class, new SafeRuleSubImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<SafeRuleSubVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, SafeRuleSubVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除保障规则子
     *
     * @param ids 需要删除的保障规则子主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
